# -*- coding: utf-8 -*-#
#
# October 16 2015, Christian Hopps <chopps@gmail.com>
#
# Copyright (c) 2015-2016, Deutsche Telekom AG.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import absolute_import, division, unicode_literals, print_function, nested_scopes
import argparse
import logging
import io
import sys
from netconf import client
from netconf import NSMAP, nsmap_update


nsmap_update({'j': "urn:TBD:params:xml:ns:yang:terastream:jdsu"})


def main (*margs):
    parser = argparse.ArgumentParser("JDSU-Scan to CSV")

    parser.add_argument("--full-125", action="store_true", help="Do 12.5GHz scan otherwise normal full.")
    parser.add_argument("--full-itu", action="store_true", help="Do ITU scan on single port.")
    parser.add_argument("--host", default="localhost", help="The host to connect to (default: localhost)")
    parser.add_argument("--port", type=int, default=9931, help="The port to connect to (default: 9931)")
    parser.add_argument("--scan-port", type=int, default=None, help="The port to scan")
    parser.add_argument("--output-prefix", default="jdsu-scan",
                        help="The Prefix of the csv file to output per port")
    parser.add_argument("--username", default="admin", help="The username to login with")
    parser.add_argument("--password", default="admin", help="The password to login with")
    parser.add_argument("-v", "--verbose", action="store_true", help="The password to login with")
    args = parser.parse_args(*margs)

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.WARNING)

    session = client.NetconfSSHSession(args.host,
                                       username=args.username,
                                       password=args.password,
                                       port=args.port,
                                       debug=args.verbose)

    try:
        if args.full_125:
            unused, reply, unused = session.send_rpc("""<full-125-scan/>""")
        elif args.full_itu:
            unused, reply, unused = session.send_rpc("""<full-itu-scan><high-resolution>true</high-resolution></full-itu-scan>""")
        else:
            unused, reply, unused = session.send_rpc("""<full-scan/>""")
    except Exception as ex:
        logging.error("Scan failed: %s", str(ex))
        sys.exit(1)

    if reply is None:
        logging.error("Scan failed")
    elif args.full_itu:
        freqlist = reply.findall("*/j:point/j:frequency", namespaces=NSMAP)
        freqlist = [ x.text for x in freqlist if x is not None and x.text]

        powerlist = reply.findall("*/j:point/j:power", namespaces=NSMAP)
        powerlist = [ x.text for x in powerlist if x is not None and x.text]

        with open(args.output_prefix + ".csv", "w") as output:
            for x, y in zip(freqlist, powerlist):
                output.write("{}\t{}\n".format(x, y))
    else:
        if args.scan_port is not None:
            start, end = args.scan_port, args.scan_port + 1
        else:
            start, end = 0, 4

        for port in range(start, end):
            xpath_prefix = "*/j:port[j:port-index='{}']/j:point/".format(port)

            freqlist = reply.findall(xpath_prefix + "j:frequency", namespaces=NSMAP)
            freqlist = [ x.text for x in freqlist if x is not None and x.text]

            powerlist = reply.findall(xpath_prefix + "j:power", namespaces=NSMAP)
            powerlist = [ x.text for x in powerlist if x is not None and x.text]

            with open(args.output_prefix + "-" + str(port) + ".csv", "w") as output:
                for x, y in zip(freqlist, powerlist):
                    output.write("{}\t{}\t{}\n".format(port, x, y))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-#
#
# October 12 2015, Christian Hopps <chopps@gmail.com>
#
# Copyright (c) 2015-2016, Deutsche Telekom AG.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import absolute_import, division, unicode_literals, print_function, nested_scopes
import logging
import os
import threading
import traceback

import netconf.util as ncutil
import netconf.error as ncerror
import netconf.server as server
from netconf import nsmap_update, NSMAP, qmap

import jdsuocm.error as jerror

nsmap_update({'j': "urn:TBD:params:xml:ns:yang:terastream:jdsu"})


logger = logging.getLogger(__name__)


class NetconfServer (object):
    NCFILTER = qmap("nc") + "filter"

    def __init__ (self, device, host_key, ssh_port=830, username=None, password=None, debug=False):
        #-----------------
        # Open the device
        #-----------------
        self.debug = debug

        host_key_path = os.path.expanduser(host_key)
        assert os.path.exists(host_key_path)

        idn = device.get_idn_data()

        # XXX Is this where we get the port count?
        assert idn[4] == "cal04" or idn[4] == "cal02"
        if idn[4] == "cal04":
            self.nports = 4
        else:
            self.nports = 1

        self.device = device
        self.device_lock = threading.Lock()

        #-----------------------
        # Start the server.
        #-----------------------

        self.controller = server.SSHUserPassController(username=username,
                                                       password=password)

        self.server = server.NetconfSSHServer(server_ctl=self.controller,
                                              port=ssh_port,
                                              host_key=host_key_path,
                                              server_methods=self,
                                              debug=debug)
        logger.info("Listening on port %d", self.server.port)

    def join (self):
        "Wait on server to terminate"
        self.server.join()

    def nc_append_capabilities (self, caps):
        ncutil.subelm(caps, "capability").text = NSMAP['j']

    def _run_device_method (self, rpc, method, *args, **kwargs):
        try:
            with self.device_lock:
                return method(*args, **kwargs)
        except jerror.OCMError as err:
            raise ncerror.RPCServerError(rpc,
                                         ncerror.RPCERR_TYPE_APPLICATION,
                                         ncerror.RPCERR_TAG_OPERATION_FAILED,
                                         message=str(err))
        except Exception as ex:
            raise ncerror.RPCServerError(rpc,
                                         ncerror.RPCERR_TYPE_APPLICATION,
                                         ncerror.RPCERR_TAG_OPERATION_FAILED,
                                         app_tag="unexpected-error",
                                         message=str(ex))

    def _rpc_param_get_frequency (self, rpc, params):
        for param in params:
            if ncutil.filter_tag_match("j:frequency", param.tag):
                break
        else:
            raise ncerror.RPCSvrMissingElement(rpc, ncutil.elm("j:frequency"))

        freq = param.strip()
        try:
            freq = int(freq.text)
            if not (190000 <= freq <= 198000):
                raise ncerror.RPCSvrBadElement(rpc, freq, message="Frequency not in range [190000, 198000]")
        except ValueError:
            raise ncerror.RPCSvrBadElement(rpc, freq, message="Frequency not an integer")

    def _rpc_param_get_boolean (self, rpc, tag, default, params):
        for param in params:
            if ncutil.filter_tag_match(tag, param.tag):
                if param is None:
                    raise ncerror.RPCSvrBadElement(rpc, param, message="invalid boolean value for " + tag)
                bval = param.text.strip().lower()
                if bval in [ "false", "no", "0" ]:
                    return False
                elif bval in [ "true", "yes", "1" ]:
                    return True
                raise ncerror.RPCSvrBadElement(rpc, param, message="invalid boolean value for " + tag)
        if default is None:
            raise ncerror.RPCSvrMissingElement(rpc, ncutil.elm(tag))
        return default

    def rpc_activate (self, unused, rpc, *params):
        # Input values
        if params:
            raise ncerror.RPCSvrErrBadMsg(rpc)

        self._run_device_method(rpc, self.device.activate)
        return ncutil.elm("ok")

    def rpc_self_test (self, unused, rpc, *params):
        # Input values
        if params:
            raise ncerror.RPCSvrErrBadMsg(rpc)

        self._run_device_method(rpc, self.device.self_test)
        return ncutil.elm("ok")

    def rpc_reset (self, unused, rpc, *params):
        # Input values
        if params:
            raise ncerror.RPCSvrErrBadMsg(rpc)

        self._run_device_method(rpc, self.device.reset)
        return ncutil.elm("ok")

    def rpc_frequency_power (self, unused, rpc, *params):
        if len(params) > 1:
            # XXX need a function to look for unknown elements and raise exc for those.
            # XXX really need a better error for not handled params
            raise ncerror.RPCSvrInvalidValue(rpc, message="Too many parameter")

        freq = self._rpc_param_get_frequency(rpc, params)
        power = self._run_device_method(rpc, self.device.get_freq_power, freq)
        result = ncutil.elm("data")
        result.append(ncutil.leaf_elm("j:power", "{:.2f}".format(power.dBm)))
        return result

    def rpc_full_itu_scan (self, unused_session, rpc, *params):
        # No input values yet
        try:
            if len(params) > 2:
                raise ncerror.RPCSvrInvalidValue(rpc, message="Too many parameters")
            # XXX Should be able to use "j:high-resolution" but it fails
            hires = self._rpc_param_get_boolean(rpc, "high-resolution", False, params)
            power_only = not self._rpc_param_get_boolean(rpc, "detect-presence", False, params)
            if power_only:
                points = self._run_device_method(rpc, self.device.get_itu_power_scan, hires)
            else:
                points = self._run_device_method(rpc, self.device.get_itu_scan, hires)

            result = ncutil.elm("data")
            for tup in points:
                ptelm = ncutil.subelm(result, "j:point")
                ptelm.append(ncutil.leaf_elm("j:frequency", tup[0]))
                if power_only:
                    ptelm.append(ncutil.leaf_elm("j:power", "{:.2f}".format(tup[1].dBm)))
                else:
                    ptelm.append(ncutil.leaf_elm("j:power", "{:.2f}".format(tup[2].dBm)))
                    ptelm.append(ncutil.leaf_elm("j:channel-presence", tup[1]))
            return result
        except jerror.OCMError as ocmerr:
            logger.error("Got OCM error in full itu scan: %s: %s", str(ocmerr),
                         traceback.format_exc())
        except Exception as err:
            logger.error("Got error in full itu scan: %s: %s", str(err),
                         traceback.format_exc())
            raise

    def _rpc_full_scan (self, method, rpc, *params):
        # No input values yet
        if params:
            logging.error("%s: _rpc_full_scan got unexpected params", str(self))
            raise ncerror.RPCSvrErrBadMsg(rpc)

        rv = self._run_device_method(rpc, method)

        result = ncutil.elm("data")
        for port, points in rv:
            portelm = ncutil.elm("j:port")
            result.append(portelm)
            portelm.append(ncutil.leaf_elm("j:port-index", port))
            for freq, power in points:
                ptelm = ncutil.subelm(portelm, "j:point")
                ptelm.append(ncutil.leaf_elm("j:frequency", freq))
                ptelm.append(ncutil.leaf_elm("j:power", "{:.2f}".format(power.dBm)))
        return result

    def rpc_full_scan (self, unused_session, rpc, *params):
        return self._rpc_full_scan(self.device.get_full_scan, rpc, *params)
    # No input values yet
        if params:
            raise ncerror.RPCSvrErrBadMsg(rpc)


    def rpc_full_125_scan (self, unused_session, rpc, *params):
        return self._rpc_full_scan(self.device.get_full_125_scan, rpc, *params)

    def rpc_get_config (self, unused_session, rpc, source_elm, unused_filter_elm):
        assert source_elm is not None
        if source_elm.find("nc:running", namespaces=NSMAP) is None:
            raise ncerror.RPCSvrMissingElement(rpc, ncutil.elm("nc:running"))

        config = ncutil.elm("data")
        for idx in range(1, 17):
            profile_elm = ncutil.elm("j:channel-profile")
            config.append(profile_elm)
            profile_elm.append(ncutil.leaf_elm("j:profile-index", idx))

            channels = self.device.get_channel_profile(idx)
            for freqs, freqe in channels:
                channel_elm = ncutil.subelm(profile_elm, "j:channel")
                range_elm = ncutil.subelm(channel_elm, "j:range")
                range_elm.append(ncutil.leaf_elm("j:frequency-start", freqs))
                range_elm.append(ncutil.leaf_elm("j:frequency-end", freqe))

        return config

    def rpc_get (self, unused_session, unused_rpc, filter_elm):
        data = ncutil.elm("data")

        get_data_methods = {
            ncutil.qname("j:ocm-type").text: self.device.get_device_type,
            ncutil.qname("j:oper-mode").text: self.device.get_oper_mode,
            ncutil.qname("j:ident-data").text: self.device.get_idn_string,
            ncutil.qname("j:device-info").text: self.device.get_module_info,
            ncutil.qname("j:application-version").text: self.device.get_app_version,
            ncutil.qname("j:temp").text: self.device.get_temp_int,
        }
        if not self.device.get_device_type().startswith("tf"):
            get_data_methods[ncutil.qname("j:safe-version").text] = self.device.get_safe_version

        infonode = ncutil.elm("j:info")
        # infonode = etree.Element(ncutil.qname("j:info"), nsmap={ 'j': NSMAP['j'] })

        # Get filter children
        children = []
        if filter_elm is not None:
            children = filter_elm.getchildren() if filter_elm is not None else []

        def get_all_values ():
            leaf_elms = []
            for key, valuef in get_data_methods.items():
                leaf_elms.append(ncutil.leaf_elm(key, valuef()))
            return leaf_elms

        # No filter children return info only
        if not children:
            ncutil.filter_leaf_values(None, infonode, get_all_values(), data)
            return data

        # Look for info filter.
        finfo = filter_elm.find("info") or filter_elm.find("j:info", namespaces=NSMAP)
        if finfo is not None:
            children = finfo.getchildren()
            if not children:
                leaf_elms = get_all_values()
            else:
                leaf_elms = []
                for felm in children:
                    tag = felm.tag
                    if tag in get_data_methods:
                        leaf_elms.append(ncutil.leaf_elm(tag, get_data_methods[tag]()))
                        rv = ncutil.filter_leaf_values(finfo, infonode, leaf_elms, data)
                        # Some selector doesn't match return empty.
            if rv is False:
                logger.error("XXX returning False")
                return data

        return data


__author__ = 'Christian Hopps'
__date__ = 'October 12 2015'
__version__ = '1.0'
__docformat__ = "restructuredtext en"



# -*- coding: utf-8 -*-#
#
# October 10 2015, Christian Hopps <chopps@gmail.com>
#
# Copyright (c) 2015-2016, Deutsche Telekom AG.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import absolute_import, division, unicode_literals, print_function, nested_scopes
import logging
import os
import pdb
import struct
from pkg_resources import Requirement, resource_filename
from sshutil.host import Host
from sshutil.conn import SSHCommandSession
from opticalutil.power import Power, Gain
from opticalutil.dwdm import frequency_to_wavelen_precise, wavelen_to_frequency
from jdsuocm.error import OCMError, get_error_result
import jdsuocm.error as jerror

logger = logging.getLogger(__name__)

from netconf import nsmap_update
nsmap_update({'j': "urn:TBD:params:xml:ns:yang:terastream:jdsu"})


"""
Message ID
Length
Command
Object ID
Instance
Parameter ID
Data Parameters                                             # up to 100/3000 words doc confused
Checksum
"""

"""
Message ID
Length
Error
Data Parameters up to 41004 words
Checksum
"""

MINRESPLEN = 2                                                # + 2 for frame
MAXRESPLEN = 41004
MINCMDLEN = 5                                                 # + 2 for frame
MAXCMDLEN = 100

TFOCM_DEFAULT_START_FREQ = 190700
TFOCM_DEFAULT_STOP_FREQ = 190700 + (128 * 50)

DEVTYPE_4PORT = 0
DEVTYPE_TFOCM = 1

"""
for 12.5GHz wide scan every 6.25Ghz, total 839 slices
1910000 -> 1962500 yields 5250.0 Ghz band

In [17]: 1962500 - 1910000
Out[17]: 52500

And there are 420 12.5GHz slices scanned twice except the final slice as
the device cannot measure the full 12.5GHz band.

In [18]: 52500 / 125
Out[18]: 420

In [19]: 52500 / 125 * 2
Out[19]: 840

"""

instance_map_4port = {
    0b1111: 0,                                                 # ALL
    0b0001: 1,                                                 # 1
    0b0010: 2,                                                 # 2
    0b0100: 3,                                                 # 3
    0b1000: 4,                                                 # 4
    0b0011: 5,                                                 # 1, 2
    0b0101: 6,                                                 # 1, 3
    0b1001: 7,                                                 # 1, 4
    0b0110: 8,                                                 # 2, 3
    0b1010: 9,                                                 # 2, 4
    0b1100: 10,                                                # 3, 4
    0b0111: 11,                                                # 1, 2, 3
    0b1011: 12,                                                # 1, 2, 4
    0b1101: 13,                                                # 1, 3, 4
    0b1110: 14                                                 # 2, 3, 4
}

inst_map = instance_map_4port

RESP_HDR = ">HHH"
RESP_0W  = RESP_HDR + "H"
RESP_1W  = RESP_0W + "H"
RESP_2W  = RESP_1W + "H"
RESP_3W  = RESP_2W + "H"
RESP_11W = RESP_HDR + "H" * 11
RESP_VAR = ""

APP_ONLY  = 0b01
SAFE_ONLY = 0b10
BOTH      = 0b11

PROFILE_ID_TAG = -1
PORT_NUM_TAG = -2
INST_MAP_TAG = -3

# Command, Object ID, Instance, Parameter ID, Has Data Params, Response Length
commands_common = {
    'START-SELF-TEST':    (1, 0, 1, 1, False, RESP_0W),
    'READ-FAIL-REG':      (2, 0, 1, 0, False, RESP_1W),
    'READ-FAIL-REG-TEMP': (2, 0, 2, 0, False, RESP_2W),
    'ACTIVATE':           (1, 1, 1, 1, False, RESP_0W),
    'DOWNLOAD-INIT':      (1, 2, 1, 1, False, RESP_0W),
    'DOWNLOAD':           (1, 3, 1, 1, True, RESP_0W),
    'GET-IDN-MSG':        (2, 4, 1, 0, False, RESP_VAR),
    'GET-MODULE-TEMP':    (2, 5, 1, 0, False, RESP_1W),
    'GET-APP-VERSION':    (2, 10, 1, 0, False, RESP_3W), # 1-port is called get firmware version for both images
    'GET-MODULE-INFO':    (2, 11, 1, 0, False, RESP_VAR),


    # XXX this is only in the 4 port!
    'RESET':              (1, 79, 1, 1, False, RESP_0W),

}

commands_1port = {
    # Only on single port
    # 6, 6, 7, 7, 8, 8
    # 1-port only
    # 21, 21 - 24, 24, 25, 26, 26, 26, 27 - 29, 31, 32, 32 - 36, 36, 37, 38, 38, 39, 40, 40, 41, 41,
    # 44, 44, 42, 43, 47, 48, 49, 51 - 54

    # data parameter 0 or 1 for low vs high res 512 words of data
    'GET-RAW-POWER-DATA':  (2, 26, 6, 0, True, RESP_VAR),
    'GET-SINGLE-POWER':    (1, 28, 1, 1, True, RESP_VAR),
    'FULL-ITU-SCAN':       (1, 31, 1, 1, True, RESP_VAR),   # This is the peak measure function
    'FULL-ITU-POWER-SCAN': (1, 37, 1, 1, True, RESP_VAR),   # Simple power function
    'SCAN-ITU-GAUSS-FIT':  (1, 27, 1, 1, True, RESP_VAR),
    'SET-FACTORY-DEFAULT': (1, 39, 1, 1, False, RESP_0W),
}
commands_4port = {
    'GET-SAFE-VERSION':   (2, 10, 2, 0, False, RESP_3W),
    'READ-PREV-CMD':      (2, 18, 4, 0, False, RESP_11W),
    'RESET':              (1, 79, 1, 1, False, RESP_0W),
    'CALIB-INIT':         (1, 14, 1, 1, False, RESP_0W),
    'CALIB-DOWNLOAD':     (1, 15, 1, 1, True, RESP_0W),
    'APPLY-CALIB':        (1, 16, 1, 1, False, RESP_0W),
    'GET-CALIB-VERSION':  (2, 17, 1, 0, False, RESP_3W),
    'SCAN':               (2, 75, INST_MAP_TAG, 0, True, RESP_VAR),
    'SET-PROFILE':        (1, 78, PROFILE_ID_TAG, 1, True, RESP_0W),
    'READ-PROFILE':       (2, 78, PROFILE_ID_TAG, 0, False, RESP_VAR),
    'FULL-SPECTRUM-SCAN': (2, 80, INST_MAP_TAG, 0, False, RESP_VAR),
    'FULL-12-SCAN':       (2, 82, INST_MAP_TAG, 0, False, RESP_VAR),
    'FULL-12-CH-SCAN':    (2, 84, INST_MAP_TAG, 0, True, RESP_VAR),
    'SET-FREQ':           (1, 85, PORT_NUM_TAG, 1, True, RESP_0W),
    'SCAN-DETECT-CHAN':   (2, 86, INST_MAP_TAG, 0, True, RESP_VAR),
    'SCAN-SPEC-DENSITY':  (2, 87, INST_MAP_TAG, 0, True, RESP_VAR),
    'SCAN-SPEC-DENSITY-CHAN': (2, 88, INST_MAP_TAG, 0, True, RESP_VAR),
}


def read_exact_len (jdsu, rlen):
    buf = b""
    blen = 0
    while blen < rlen:
        leftover = rlen - blen
        nbuf = jdsu.recv(leftover)
        nblen = len(nbuf)
        assert nblen <= leftover
        buf += nbuf
        blen += nblen
    return buf


def read_var_resp (jdsu):
    hdr = read_exact_len(jdsu, 6)
    if len(hdr) != 6:
        pdb.set_trace()
    msgid, mlen, result = struct.unpack('>HHH', hdr)
    assert mlen >= 2
    mlen -= 1
    rest = read_exact_len(jdsu, mlen * 2)
    if mlen > 1:
        data = rest[:-2]
    else:
        data = ""

    cksum = struct.unpack(">H", rest[-2:])[0]
    oursum = sum(unpack_data_words(rest[:-2])) + msgid + mlen + 1 + result
    oursum &= 0xFFFF
    if oursum != cksum:
        logger.error("BAD CKSUM: %s %s", str(unpack_data_words(hdr)), str(unpack_data_words(rest)))
        logger.error("BAD CKSUM: ours: %d theres: %d", oursum, cksum)

    return msgid, result, data


def instance_to_ports (bits):
    ports = []
    port = 0
    while bits:
        if bits & 0x1:
            ports.append(port)
        bits = (bits >> 1)
        port += 1
    return ports


def unpack_data_words (wordstring):
    wlen = len(wordstring)
    assert (wlen % 2) == 0
    wlen = wlen // 2
    return struct.unpack(">" + str(wlen) + "H", wordstring)


def unpack_data_string (wordstring):
    # XXX look for NUL?

    nullidx = wordstring.find(0)
    if nullidx == -1:
        nullidx = len(wordstring)
    return wordstring[:nullidx].decode('ascii')


def unpack_signed_unsigned(data):
    nval = len(data) // 2
    spoints = struct.unpack(">" + str(nval) + "h", data)
    upoints = struct.unpack(">" + str(nval) + "H", data)
    return spoints, upoints


def unpack_signed(data):
    nval = len(data) // 2
    spoints = struct.unpack(">" + str(nval) + "h", data)
    return spoints


def unpack_unsigned(data):
    nval = len(data) // 2
    upoints = struct.unpack(">" + str(nval) + "H", data)
    return upoints


def get_next_msgid ():
    this_id = get_next_msgid.next
    get_next_msgid.next += 1
    return this_id
get_next_msgid.next = 1


def send_cmd(jdsu, commands, cmdname, data=b"", instance=None, debug=False):
    cmdinfo = commands[cmdname]
    cmd = list(cmdinfo[0:4])
    if cmd[2] == INST_MAP_TAG:
        assert instance is not None
        cmd[2] = inst_map[instance]
    elif cmd[2] == PROFILE_ID_TAG:
        assert instance is not None
        cmd[2] = instance
    elif cmd[2] == PORT_NUM_TAG:
        assert instance is not None
        cmd[2] = instance

    if data:
        assert cmdinfo[4]
    else:
        assert not cmdinfo[4]
    dlen = len(data)
    assert (dlen % 2) == 0
    dwlen = dlen // 2
    clen = len(cmd) + dwlen + 1                             # +1 cksum
    rawcmd = [ get_next_msgid(), clen ] + list(cmd)
    cksum = sum(rawcmd)
    rawdata = struct.pack(">6H", *rawcmd)
    if data:
        cksum += sum(unpack_unsigned(data))
    rawdata += data + struct.pack(">H", cksum & 0xFFFF)
    rlen = len(rawdata) // 2
    assert clen == rlen - 2
    if debug:
        logger.debug("sending: %s", str(cmdname))
        logger.debug("sending: %s", str(unpack_unsigned(rawdata)))
    jdsu.send(rawdata)


class OCM (object):
    def __init__ (self, device, debug=False):
        self.device = device
        self.debug = debug
        assert not self.drain_serial_read_queue()

        # single port safe: [u'JDSU', u'TFOCM', u'50GHz', u'safe00.04.68']
        # single port app:  [u'JDSU', u'TFOCM', u'50GHz', u'hw46', u'cal02', u'appfw03.08.94']

        self.commands = dict(commands_common.items())

        idn = self.get_idn_data()
        if len(idn) > 4 and idn[4] == 'cal04':
            self.devtype = DEVTYPE_4PORT
            self.nports = 4
            self.commands.update(commands_4port.items())
        else:
            self.devtype = DEVTYPE_TFOCM
            self.nports = 1
            self.commands.update(commands_1port.items())

        # reset the device on init, only works on 4 port.
        # For tf-ocm though we reset to factory default a poor man's reset
        if not self.is_safe_mode(idn):
            # Reset the device on open.
            self.reset()

        # Activate to application software if we are safe mode
        idn = self.get_idn_data()
        if self.is_safe_mode(idn):
            self.activate()

        # Add in device specific commands
        self.self_test()

        # # print(self.get_module_info())
        # print(str(self.get_temp()))
        # # print(self.get_safe_version())
        # print(self.get_app_version())

        # # unused = self.get_raw_power_scan()
        # # for freq in [194150, 194100, 194050, 194000, 193950, 193900, 193850]:
        # for freq in range(193850, 194050, 1):
        #     # power = self.get_freq_power(freq)
        #     # print("Power for {}: {}".format(freq, power))

        #     wl = frequency_to_wavelen_precise(freq)
        #     power = self.get_wavelen_power(wl)
        #     print("{} {}".format(freq, power))

        # for wl in [1544.92, 1545.32, 1545.72]:

    def run_cmd_status(self, cmdname, data=b"", instance=None):
        if cmdname not in self.commands:
            return jerror.EBADCMD, b""

        # XXX really need to catch any errors here and return an error instead to match API.

        try:
            send_cmd(self.device, self.commands, cmdname, data, instance, debug=self.debug)
        except AssertionError:
            return jerror.EBADCMD

        respfmt = self.commands[cmdname][5]
        if not respfmt:
            unused, error, data = read_var_resp(self.device)
        else:
            resplen = (len(respfmt) - 1) * 2
            rdata = read_exact_len(self.device, resplen)
            unused_msgid, unused_mlen, error = struct.unpack('>HHH', rdata[:6])
            data = rdata[6:-2]
            cksum = rdata[-2:]
            # XXX check cksum
            # XXX verify len
        if error:
            logging.debug("Command %s failed with result: %s", cmdname, get_error_result(error))

        return error, data

    def run_cmd(self, cmdname, data=b"", instance=None):
        error, data = self.run_cmd_status(cmdname, data, instance)
        if error:
            raise OCMError(error)
        return data

    def drain_serial_read_queue (self):
        while self.device.recv_ready():
            extra = self.device.recv()
            if extra:
                print("Got {} extra leftover bytes".format(len(extra)))

    def is_safe_mode (self, idn=None):
        if idn is None:
            idn = self.get_idn_data()
        for elm in idn:
            if "safe" in elm.lower():
                return True
        return False

    def set_factory_default (self):
        "Set all configurable paramters to factory defaults"
        self.run_cmd("SET-FACTORY-DEFAULT")

    def reset (self):
        if self.devtype != DEVTYPE_4PORT:
            return self.set_factory_default()
        self.run_cmd("RESET")
        assert not self.drain_serial_read_queue()

    def activate (self):
        self.run_cmd("ACTIVATE")

    def get_idn_string (self):
        # IDN is made of of ASCII characters in 16 bit values.
        wordstring = self.run_cmd("GET-IDN-MSG")
        wlen = len(wordstring)
        assert (wlen % 2) == 0
        wlen = wlen // 2
        data = "".join([ chr(x) for x in struct.unpack(">" + str(wlen) + "H", wordstring) ])
        return data.encode('ascii').decode('ascii')

    def get_idn_data (self):
        return self.get_idn_string().split(",")

    def get_device_type (self):
        types = {
            DEVTYPE_4PORT: "ocm-4-port",
            DEVTYPE_TFOCM: "tf-ocm-1-port",
        }
        try:
            return types[self.devtype]
        except KeyError:
            return "unknown"

    def get_oper_mode (self):
        idn = self.get_idn_data()
        oper_mode = "safe-mode" if idn[2] == "SafeImage" else "application-mode"
        return oper_mode

    def self_test (self):
        self.run_cmd("START-SELF-TEST")

    def get_safe_version (self):
        assert self.devtype == DEVTYPE_4PORT
        return "{}.{}.{}".format(*unpack_data_words(self.run_cmd("GET-SAFE-VERSION")))

    def get_app_version (self):
        return "{}.{}.{}".format(*unpack_data_words(self.run_cmd("GET-APP-VERSION")))

    def get_fail_reg (self):
        return unpack_data_words(self.run_cmd("READ-FAIL-REG"))[0]

    def get_fail_reg_temp (self):
        data = unpack_data_words(self.run_cmd("READ-FAIL-REG-TEMP"))
        return data[0], data[1] / 10

    def get_temp (self):
        return unpack_data_words(self.run_cmd("GET-MODULE-TEMP"))[0] / 10

    def get_temp_int (self):
        return unpack_data_words(self.run_cmd("GET-MODULE-TEMP"))[0]

    def get_module_info (self):
        data = unpack_data_words(self.run_cmd("GET-MODULE-INFO"))
        import pdb
        pdb.set_trace()
        return data

    def get_raw_power_scan (self, hires=False):
        assert self.devtype == DEVTYPE_TFOCM
        resolution = 2 if hires else 1
        data = self.run_cmd("GET-RAW-POWER-DATA", struct.pack(">H", resolution))
        spoints = unpack_signed(data)
        return spoints

    def get_freq_power (self, freq):
        "Get power level of a frequency in GHz"
        assert self.devtype == DEVTYPE_TFOCM
        val = int(frequency_to_wavelen_precise(int(freq)) * 1000)
        msw = (val >> 16) & 0xFFFF
        lsw = (val & 0xFFFF)
        # This seems to ignore our resolution request and always returns low
        data = self.run_cmd("GET-SINGLE-POWER", struct.pack(">HHHH", 1, msw, lsw, 2))
        power = Power(unpack_signed(data)[0] / 10)
        return power

    def get_wavelen_power (self, wavelen):
        "Get power level of a wavelen in nanometers (non-int ok)"
        assert self.devtype == DEVTYPE_TFOCM
        val = int(wavelen * 1000)
        msw = (val >> 16) & 0xFFFF
        lsw = (val & 0xFFFF)
        data = self.run_cmd("GET-SINGLE-POWER", struct.pack(">HHHH", 2, msw, lsw, 2))

        power = Power(unpack_signed(data)[0] / 100)
        return power

    def get_channel_profile (self, profile_id):
        assert self.devtype == DEVTYPE_4PORT
        data = self.run_cmd("READ-PROFILE", instance=profile_id)
        nchan = struct.unpack(">H", data[:2])[0]
        data = data[2:]
        if len(data) != nchan * 4:
            raise ValueError("Frequencies returned {} different from expected {}".format(len(data) // 2, nchan * 2))
        freqlist = [ x + 1900000 for x in unpack_unsigned(data) ]
        if len(freqlist) % 1 != 0:
            raise ValueError("Channel frequencies not paired, total frequency count {}".format(len(freqlist)))
        freqlist = zip(freqlist[::2], freqlist[1::2])
        return freqlist

    def get_itu_scan (self, hires):
        assert self.devtype == DEVTYPE_TFOCM
        hival = 2 if hires else 1
        data = self.run_cmd("FULL-ITU-SCAN", struct.pack(">H", hival))
        if hires:
            size = 512
            # XXX this is probably msw lsw of channel wavelen
            words = unpack_unsigned(data[:size])
            it = iter(words)
            uints = [ (((msw & 0xFFFF) << 16) + (lsw & 0xFFFF)) for msw, lsw in zip(it, it) ]
            frequency = [ wavelen_to_frequency(float(x) / 1000.0) if x > 0 else 0 for x in uints ]
            # frequency = [ float(x) / 1000.0 if x > 0 else 0 for x in uints ]
        else:
            size = 256
            # frequency = [ wavelen_to_frequency(((x / 100.0) + 1500) if x > 0 else 0 for x in unpack_unsigned(data[:size]) ]
            frequency = [ wavelen_to_frequency((float(x) / 100.0) + 1500.0) for x in unpack_unsigned(data[:size]) ]
            # frequency = [ (float(x) / 100.0) + 1500.0 for x in unpack_unsigned(data[:size]) ]
            # frequency = unpack_unsigned(data[:size])

        data = data[size:]
        presence = unpack_unsigned(data[:256])
        data = data[256:]
        if hires:
            power = [ Power(x / 100) for x in unpack_signed(data[:256]) ]
        else:
            power = [ Power(x / 10) for x in unpack_signed(data[:256]) ]
        return zip(frequency, presence, power)

    def get_itu_power_scan (self, hires):
        assert self.devtype == DEVTYPE_TFOCM
        hival = 2 if hires else 1
        data = self.run_cmd("FULL-ITU-POWER-SCAN", struct.pack(">H", hival))
        if hires:
            power = [ Power(x / 100) for x in unpack_signed(data[:256]) ]
        else:
            power = [ Power(x / 10) for x in unpack_signed(data[:256]) ]

        # XXX Are the start and stop frequency or the interval affected by the user settings?
        # or always constant b/c it's the ITU variant of the commands
        return zip(range(TFOCM_DEFAULT_START_FREQ, TFOCM_DEFAULT_STOP_FREQ + 1, 50),
                   power)

    def get_full_scan (self, instance=0b1111):
        assert self.devtype == DEVTYPE_4PORT
        data = self.run_cmd("FULL-SPECTRUM-SCAN", instance=instance)

        nports = struct.unpack(">H", data[0:2])[0]
        data = data[2:]
        if self.debug:
            logger.debug("FSCAN: Port Count: %d", nports)

        result = []
        for port in instance_to_ports(instance):
            npoints = struct.unpack(">H", data[:2])[0]
            data = data[2:]

            spoints, upoints = unpack_signed_unsigned(data[:4 * npoints])
            data = data[4 * npoints:]

            if self.debug:
                logger.debug("FSCAN PORT %s points %d", port, npoints)

            rpoints = []
            for freq, power in zip(upoints[::2], spoints[1::2]):
                power = Power(power / 100) + Gain(20)       # Tap is 1% so add 20dB
                freq = 1900000 + freq
                rpoints.append((freq, power))

            lrpoints = len(rpoints)
            if npoints != lrpoints:
                raise ValueError("Returned points {} different from expected {}".format(lrpoints, npoints))

            result.append((port, rpoints))

        # Want better error here.
        if data:
            raise ValueError("Extra data form OCM of len: {}".format(len(data)))

        return result

    def get_full_125_scan (self, instance=0b1111):
        assert self.devtype == DEVTYPE_4PORT
        data = self.run_cmd("FULL-12-SCAN", instance=instance)

        nports = struct.unpack(">H", data[0:2])[0]
        data = data[2:]
        if self.debug:
            logger.debug("FSCAN125x625: Port Count: %s", str(nports))

        result = []
        for port in instance_to_ports(instance):
            npoints = struct.unpack(">H", data[:2])[0]
            data = data[2:]
            if npoints != 839:
                raise ValueError("Too man points %d (not 839) in 12.5x6.25 scan", npoints)

            spoints = struct.unpack(">" + str(npoints) + "h", data[:2 * npoints])
            data = data[2 * npoints:]

            if self.debug:
                logger.debug("FSCAN125x625 PORT %d points %d", port, npoints)

            rpoints = []
            # Start of scan range
            freq = 191000
            for power in spoints:
                rpoints.append((freq, Power(power / 100) + Gain(20)))
                freq += 6.25

            lrpoints = len(rpoints)
            if npoints != lrpoints:
                raise ValueError("Returned points {} different from expected {}".format(lrpoints, npoints))

            result.append((port, rpoints))

        if data:
            raise ValueError("Extra data form OCM of len: {}".format(len(data)))

        return result

    def dump_channel_scan (self):
        assert self.devtype == DEVTYPE_4PORT
        data = struct.pack(">HHHH", 2, 2, 2, 2)
        data = self.run_cmd("FULL-12-CH-SCAN", data=data, instance=0b1111)

        nports = struct.unpack(">H", data[0:2])[0]
        data = data[2:]
        print("FSCAN: Port Count: {}".format(nports))

        port = 1
        while data:
            nchan = struct.unpack(">H", data[:2])[0]
            data = data[2:]
            print("FSCAN PORT {} Channel count {}".format(port, nchan))

            chandata = data[:nchan * 3 * 2]
            data = data[nchan * 3 * 2:]
            spoints = struct.unpack(">" + str(nchan * 3) + "h", chandata)
            upoints = struct.unpack(">" + str(nchan * 3) + "H", chandata)
            with open("fpcs-{}.csv".format(port), "w") as f:
                for freq, power, present in zip(upoints[::3], spoints[1::3], upoints[2::3]):
                    power = Power(power / 100) + Gain(20)
                    freq = (1900000 + freq) / 10
                    if True or power > Power(-25):
                        print("FSCAN PORT {} CHFRQ: {}\tPWR: {}\tPRES: {}".format(port,
                                                                                  freq,
                                                                                  power,
                                                                                  present))
                    f.write("{}\t{}\n".format(freq, power))
            npoints = struct.unpack(">H", data[:2])[0]
            data = data[2:]

            pdata = data[:npoints * 2]
            data = data[2 * npoints:]
            spoints = struct.unpack(">" + str(npoints) + "h", pdata)

            # print("FSCAN PORT {} points {}".format(port, npoints))
            # for idx, power in enumerate(spoints):
            #     power = Power(power / 100) + Gain(20)
            #     if power > Power(-30):
            #         print("FSCAN PORT {}: SLICE IDX: {}\tPWR: {}".format(port,
            #                                                              idx,
            #                                                              power))
            port += 1

    def dump_spectral_density (self):
        assert self.devtype == DEVTYPE_4PORT
        data = struct.pack(">HHHH", 2, 2, 2, 2)
        data = self.run_cmd("SCAN-SPEC-DENSITY", data=data, instance=0b1111)

        nports = struct.unpack(">H", data[0:2])[0]
        data = data[2:]
        print("FSCAN: Port Count: {}".format(nports))

        port = 1
        while data:
            npoints = struct.unpack(">H", data[:2])[0]
            data = data[2:]
            spoints = struct.unpack(">" + str(npoints * 3) + "h", data[:6 * npoints])
            upoints = struct.unpack(">" + str(npoints * 3) + "H", data[:6 * npoints])
            data = data[6 * npoints:]
            print("SPDENSE PORT {} points {}".format(port, npoints))
            with open("sd-{}.csv".format(port), "w") as f:
                for freq, avgpower, maxpower in zip(upoints[::3], spoints[1::3], spoints[2::3]):
                    avgpower = Power(avgpower / 100) + Gain(20)
                    maxpower = Power(maxpower / 100) + Gain(20)
                    freq = (1900000 + freq) / 10
                    print("SPDENSE PORT {}: FRQ: {}\tWVL: "
                          "{}\t Avg Pwr: {}\tMax Pwr: {}".format(port,
                                                                 freq,
                                                                 frequency_to_wavelen(freq),
                                                                 avgpower,
                                                                 maxpower))
                    f.write("{}\t{}\t{}\n".format(freq, avgpower, maxpower))
            port += 1

    def read_channel_profile (self):
        assert self.devtype == DEVTYPE_4PORT
        # XXX errors?
        for profile_id in range(1, 16):
            pass

    def write_channel_profile (self):
        assert self.devtype == DEVTYPE_4PORT
        freqlist = []
        nchan = 0
        for freq in range(11000, 61000, 1000):
            freqlist.append(freq - 125)
            freqlist.append(freq + 125)
            nchan += 1

        data = struct.pack(">H", nchan)
        data += struct.pack(">" + str(nchan * 2) + "H", *freqlist)
        self.run_cmd("SET-PROFILE", data=data, instance=2)


class LocalOCM (OCM):
    def __init__ (self, devname, debug=False):
        import serial
        self.serial = serial.Serial(port=devname,
                                    timeout=0,
                                    baudrate=115200,
                                    parity=serial.PARITY_NONE,
                                    stopbits=serial.STOPBITS_ONE,
                                    xonxoff=False,
                                    rtscts=False,
                                    bytesize=serial.EIGHTBITS)
        # if debug:
        #     sys.stderr.write("sercat: Opening serial\n")
        # #syslog.syslog("sercat: Opening serial\n")
        # ### serial.open()
        # serial.nonblocking()
        # if debug:
        #     sys.stderr.write("sercat: Opened serial\n")
        # #syslog.syslog("sercat: Opened serial\n")
        super(LocalOCM, self).__init__(self.serial, debug=debug)


class RemoteOCM (OCM):
    def __init__ (self, jdsu_host, devname, username=None, password=None, debug=False):
        self.host = jdsu_host

        # Copy latest sercat
        sercat_path = resource_filename(Requirement.parse("jdsuocm"), "jdsuocm/sercat.py")
        assert os.path.exists(sercat_path)
        rhost = Host(jdsu_host, username=username, password=password, debug=debug)
        rhost.copy_to(sercat_path, "./sercat.py")

        # Open the connection to the serial port.
        self.session = SSHCommandSession(jdsu_host,
                                         22,
                                         "/usr/bin/python -u sercat.py " + devname,
                                         username=username,
                                         password=password)
        self.log_sercat_stderr()                            # Start the stderr logger.

        super(RemoteOCM, self).__init__(self.session, debug=debug)

    def log_sercat_stderr (self):
        import threading

        def threadmain ():
            data = ""
            while True:
                data += self.session.recv_stderr()
                try:
                    while True:
                        idx = data.index("\n")
                        logger.warn("SERCAT (%s) STDERR: %s", self.host, data[:idx])
                        data = data[idx + 1:]
                except ValueError:
                    break

        logger_thread = threading.Thread(target=threadmain)
        logger_thread.daemon = True
        logger_thread.start()


__author__ = 'Christian Hopps'
__date__ = 'October 10 2015'
__version__ = '1.0'
__docformat__ = "restructuredtext en"



#!/usr/bin/python -u
# -*- coding: utf-8 -*-#
#
# Copyright (c) 2015 by Christian E. Hopps
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
#; distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import absolute_import, division, print_function, nested_scopes
import select
import serial
import sys
import syslog


def main (devname, debug=False):
    serdev = serial.Serial(port=devname,
                           timeout=0,
                           baudrate=115200,
                           parity=serial.PARITY_NONE,
                           stopbits=serial.STOPBITS_ONE,
                           xonxoff=False,
                           rtscts=False,
                           bytesize=serial.EIGHTBITS)
    if debug:
        sys.stderr.write("sercat: Opening serial\n")
    #syslog.syslog("sercat: Opening serdev\n")
    ### serial.open()
    serdev.nonblocking()
    if debug:
        sys.stderr.write("sercat: Opened serial\n")
    #syslog.syslog("sercat: Opened serial\n")

    # try:
    #    import fcntl
    #    fcntl.fcntl(sys.stdin.fileno(), fcntl.F_SETFL, os.O_NONBLOCK)
    # except Exception as error:
    #    syslog.syslog("sercat: error with fcntl: %s\n", str(error))

    try:
        while True:
            syslog.syslog("sercat: selecting\n")
            readfds, unused, unused = select.select([ sys.stdin, serdev ], [], [])
            if sys.stdin in readfds:
                if debug:
                    sys.stderr.write("sercat: Stdin read ready\n")
                # syslog.syslog("sercat: Stdin read ready\n")
                # inbuf = sys.stdin.read(1)
                inbuf = sys.stdin.read(1)
                if not inbuf:
                    # syslog.syslog("sercat: Exiting cleanly due to EOF on stdin")
                    if debug:
                        sys.stderr.write("Exiting due to EOF on stdin\n")
                    sys.exit(0)
                if debug:
                    sys.stderr.write("sercat: Stdin read {} bytes\n".format(len(inbuf)))
                # syslog.syslog("sercat: Stdin read {} bytes\n".format(len(inbuf)))
                outcount = serdev.write(inbuf)
                if debug:
                    sys.stderr.write("sercat: Serial wrote {} bytes\n".format(outcount))
                # syslog.syslog("sercat: Serial wrote {} bytes\n".format(outcount))
                assert outcount == len(inbuf)
            if serdev in readfds:
                if debug:
                    sys.stderr.write("sercat: Serial read ready\n")
                # syslog.syslog("sercat: Serial read ready\n")
                n = serdev.inWaiting()
                if not n:
                    # syslog.syslog("sercat: Error read ready on serial but no chars!")
                    if debug:
                        sys.stderr.write("Error read ready on serial but no chars!\n")
                    sys.exit(1)
                else:
                    if debug:
                        sys.stderr.write("sercat: Serial reading {} bytes\n".format(n))
                    # syslog.syslog("sercat: Serial reading {} bytes\n".format(n))
                    outbuf = serdev.read(n)
                    if debug:
                        sys.stderr.write("sercat: Serial read {} bytes\n".format(len(outbuf)))
                    # syslog.syslog("sercat: Serial read {} bytes\n".format(len(outbuf)))
                    sys.stdout.write(outbuf)
                    if debug:
                        sys.stderr.write("sercat: Stdout wrote {} bytes\n".format(len(outbuf)))
                    # syslog.syslog("sercat: Stdout wrote {} bytes\n".format(len(outbuf)))
    except Exception as error:
        syslog.syslog("sercat: error in try: %s\n", str(error))


if __name__ == "__main__":
    # Could process all the serial settings here..
    main(sys.argv[1])



# -*- coding: utf-8 -*-#
#
# October 11 2015, Christian Hopps <chopps@gmail.com>
#
# Copyright (c) 2015-2016, Deutsche Telekom AG.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import absolute_import, division, unicode_literals, print_function, nested_scopes

#--------------------
# Error Result Codes
#--------------------

ENOERR = 0
EBADCMD = 1
EVALRANGE = 2
EFAILED = 3
EPERM = 4
EBADCALIB = 5
EFIRMSIZE = 6
EMODTYPE = 7
EFIRMCKSUM = 8
EFRAMECKSUM = 9
EPROTOCOL = 10
ESUCCESSHARDFAIL = 11
EFAILHARDFAIL = 12
error_string = {
    ENOERR: "ENOERR",
    EBADCMD: "EBADCMD",
    EVALRANGE: "EVALRANGE",
    EFAILED: "EFAILED",
    EPERM: "EPERM",
    EBADCALIB: "EBADCALIB",
    EFIRMSIZE: "EFIRMSIZE",
    EMODTYPE: "EMODTYPE",
    EFIRMCKSUM: "EFIRMCKSUM",
    EFRAMECKSUM: "EFRAMECKSUM",
    EPROTOCOL: "EPROTOCOL",
    ESUCCESSHARDFAIL: "ESUCCESSHARDFAIL",
    EFAILHARDFAIL: "EFAILHARDFAIL",
}

#--------------------
# HW Failure Bitmask
#--------------------

EHW_TEMP = 0x2
EHW_CALIB = 0x4
EHW_APP_CORRUPT = 0x8
EHW_SCAN = 0x10
EHW_OTHER = 0x10
EHW_FLASH_FAIL = 0x80


def get_error_result (result):
    if result not in error_string:
        return "Unknown error: " + str(result)
    return error_string[result]


class OCMError (Exception):
    def __init__ (self, error):
        self.error = error
        self.errstr = get_error_result(error)
        super (OCMError, self).__init__(self.errstr)


__author__ = 'Christian Hopps'
__date__ = 'October 11 2015'
__version__ = '1.0'
__docformat__ = "restructuredtext en"



# -*- coding: utf-8 -*-#
#
# Copyright (c) 2015-2016, Deutsche Telekom AG.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
from setuptools import setup

required = [
    "netconf",
    "opticalutil",
    "pyserial",
    "sshutil"
]


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup (name='jdsuocm',
       version='0.2.0',
       description='JDSU OCM',
       long_description=read("README.rst"),
       author='Christian E. Hopps',
       author_email='chopps@gmail.com',
       license='Apache License, Version 2.0',
       install_requires=required,
       url='https://github.com/choppsv1/jdsu-ocm',
       entry_points={ "console_scripts": [ "jdsu-scan = jdsuocm.scan:main",
                                           "jdsu-server = jdsuocm.main:main" ]},
       packages=['jdsuocm'])



# -*- coding: utf-8 -*-#
#
# October 3 2015, Christian Hopps <chopps@gmail.com>
#
# Copyright (c) 2015-2016, Deutsche Telekom AG.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import absolute_import, division, unicode_literals, print_function, nested_scopes
import argparse
import logging
import os
import sys
from paramiko import RSAKey
import jdsuocm.device as device
import jdsuocm.server as server


def main (*margs):
    parser = argparse.ArgumentParser("JDSU-OCM Netconf Server")

    parser.add_argument("--server-port", type=int, default=830, help="Port for netconf to listen on")
    parser.add_argument("--server-username", default="admin", help="Netconf user")
    parser.add_argument("--server-password", default="admin", help="Netconf password")
    parser.add_argument("--server-host-key", help="Server SSH Host Key")
    parser.add_argument("--device-host", help="The remote JDSU host to run sercat on otherwise local")
    parser.add_argument("--device-name", default="/dev/ttyUSB0", help="The serial port device")
    parser.add_argument("--device-username", help="The username to login with")
    parser.add_argument("--device-password", help="The password to login with")
    parser.add_argument("--device-key", help="SSH Private key to use")
    parser.add_argument("-v", "--verbose", action="store_true", help="Verbose logging")
    parser.add_argument("-d", "--debug", action="store_true", help="Enable debug logging")
    args = parser.parse_args(*margs)

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    elif args.verbose:
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.WARNING)
    logger = logging.getLogger(__name__)

    # Mutually exclusive
    if args.device_password and args.device_key:
        logger.critical("Can't specify both --device-key and --device-password")
        sys.exit(1)

    if not args.server_host_key or not os.path.exists(os.path.expanduser(args.server_host_key)):
        logger.critical("Server host ssh key required.")
        sys.exit(1)

    if not args.device_host:
        jdsu = device.LocalOCM(args.device_name, debug=args.debug)
    else:
        if args.device_key:
            password = RSAKey.from_private_key_file(args.device_key)
        else:
            password = args.device_password
        jdsu = device.RemoteOCM(args.device_host,
                                args.device_name,
                                username=args.device_username,
                                password=password,
                                debug=args.debug)

    ncserver = server.NetconfServer(jdsu,
                                    args.server_host_key,
                                    ssh_port=args.server_port,
                                    username=args.server_username,
                                    password=args.server_password,
                                    debug=args.debug)
    ncserver.join()

if __name__ == "__main__":
    main()


# print(etree.tounicode(jdsu.get_full_scan(), pretty_print=True))
# jdsu.dump_full_125_scan()
# jdsu.dump_channel_scan()
# jdsu.dump_spectral_density()
# jdsu.write_channel_profile()
# jdsu.read_channel_profile()

# XXX add to delete?
# assert not drain_serial_read_queue(jdsu)
